/*
   Wuerfel

  created 2019
  by Marco Janßen

  License:
  Attribution-NonCommercial-ShareAlike 4.0 International
  (CC BY-NC-SA 4.0)
  https://creativecommons.org/licenses/by-nc-sa/4.0/
*/


p5.disableFriendlyErrors = true; // disables FES


const GRAVITY = -1.2;
const BOUNCE = 0.65;
const REST = 0.96;

let dice_size;
let depth;




function setup() {
  createCanvas(windowWidth, windowHeight, WEBGL);
	setAttributes('antialias', true);
	ellipseMode(CENTER); 
	rectMode(CENTER);
	angleMode(DEGREES);
	
	frameRate(60);
	resizeLayout();
}


function windowResized() {
  resizeCanvas(windowWidth, windowHeight);
	resizeLayout();
} 


function resizeLayout() {
	dice_size = (width + height) * 0.1;
	depth = dice_size * 4;
	
	dice1 = new Dice(-dice_size, 0, depth, dice_size, color(0, 160 ,0), 220, [1,2,3,4,5,6]);
	dice2 = new Dice(dice_size, 0, depth, dice_size, color(160, 0 , 160), 220, [1,2,4,5,6]);
	
	strokeWeight(2);
}
	

function draw() {
	background(230);
	push();
	translate(0, 0, -depth - dice_size * 0.5);
	fill(160);
	rect(0, 0, width + dice_size, height + dice_size);
	fill(80);
	noStroke();
	ellipse(dice1.pos.x, dice1.pos.y, map(dice1.pos.z, 0, -depth, dice_size * 0.5, dice_size * 1.4));
	ellipse(dice2.pos.x, dice2.pos.y, map(dice2.pos.z, 0, -depth, dice_size * 0.5, dice_size * 1.4));
	pop();
	
	push();
	translate(0, -(height + dice_size) * 0.5, -depth * 0.5);
	rotateX(90);
	fill(245);
	rect(0, 0, width + dice_size, height + dice_size * 4);
	pop();
	
	push();
	translate(0, (height + dice_size) * 0.5, -depth * 0.5);
	rotateX(-90);
	fill(210);
	rect(0, 0, width + dice_size, height + dice_size * 4);
	pop();
	
	dice1.update();
	dice1.render();
	
	dice2.update();
	dice2.render();
	
	dice1.bounce(dice2);
}


function touchMoved() {
	dice1.drag(mouseX - width * 0.5 - dice_size, mouseY - height * 0.5);
	
	dice2.drag(mouseX - width * 0.5 + dice_size, mouseY - height * 0.5);
	
  return false;
}

function mouseDragged() {
	dice1.drag(mouseX - width * 0.5 - dice_size, mouseY - height * 0.5);
	
	dice2.drag(mouseX - width * 0.5 + dice_size, mouseY - height * 0.5);
}

function mousePressed() {
	dice1.reset(mouseX - width * 0.5 - dice_size, mouseY - height * 0.5);
	
	dice2.reset(mouseX - width * 0.5 + dice_size, mouseY - height * 0.5);
	
	return false;
}


function mouseReleased() {
	dice1.release();
	
	dice2.release();
}



class Dice {
	constructor(_x, _y,_height, _size, _c_bg, _c_dot, numbers) {
		this.pos = {
			x : _x,
			y : _y,
			z : 0
		}
		this.start_x = _x;
		this.start_y = _y;
		this.height = -_height;
		this.vel = {
			x : random(-10, 10),
			y : random(-10, 10),
			z : 0
		}
		this.rot = {
			x : 0,
			y : 0
		}
		this.orientation = {
			x : 0,
			y : 0
		}
		this.size = _size;
		this.c_bg = _c_bg;
		this.c_dot = _c_dot;
		this.released = true;
		this.numbers = numbers;
	}
	
	reset(_x, _y) {
		this.pos.x = _x;
		this.pos.y = _y;
		this.pos.z = 0;
		this.vel.x = 0;
		this.vel.y = 0;
		this.vel.z = 0;
		this.rot.x = 0;
		this.rot.y = 0;
		let rand = random(this.numbers);
		let wx;
		let wy;

		switch(rand) {
			case 1:
				wx = 0;
				wy = 0;
				break;
			case 2:
				wx = 3;
				wy = 0;
				break;
			case 3:
				wx = 0;
				wy = 3;
				break;
			case 4:
				wx = 0;
				wy = 1;
				break;
			case 5:
				wx = 1;
				wy = 0;
				break;
			case 6:
				wx = 2;
				wy = 0;
				break;
		}

		this.orientation.x = wx * 90 + random(-45, 45);
		this.orientation.y = wy * 90 + random(-45, 45);

		this.released = false;
	}
	
	render() {
		push();
		translate(this.pos.x, this.pos.y, this.pos.z);
		
		let theta = sqrt((this.rot.x * this.rot.x) + (this.rot.y * this.rot.y));
		let norm;
		theta > 0 ? norm = 1 / theta : norm = 1;
		
		let l = this.rot.x * norm;
		let m = this.rot.y * norm;
		let n = 0;

		let mct = 1 - cos(theta);
		let ct = cos(theta);
		let st = sin(theta);

		applyMatrix(
			l*l*(mct)+ct, m*l*(mct)-n*st, n*l*(mct)+m*st, 0.0,
			l*m*(mct)+n*st, m*m*(mct)+ct, n*m*(mct)-l*st, 0.0,
			l*n*(mct)-m*st, m*n*(mct)+l*st, n*n*(mct)+ct, 0.0,
			0.0, 0.0, 0.0, 1.0);
			
		
		rotateX(this.orientation.x);
		
		theta = this.orientation.y;
		
		l = 0;
		m = cos(this.orientation.x);
		n = -sin(this.orientation.x);

		mct = 1 - cos(theta);
		ct = cos(theta);
		st = sin(theta);

		applyMatrix(
			l*l*(mct)+ct, m*l*(mct)-n*st, n*l*(mct)+m*st, 0.0,
			l*m*(mct)+n*st, m*m*(mct)+ct, n*m*(mct)-l*st, 0.0,
			l*n*(mct)-m*st, m*n*(mct)+l*st, n*n*(mct)+ct, 0.0,
			0.0, 0.0, 0.0, 1.0);
		
		stroke(0);
		this.released ? strokeWeight(1) : strokeWeight(2);
		
		push();
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(0, 0, this.size * 0.2);
		pop();

		push();
		rotateX(90);
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(-this.size * 0.2, -this.size * 0.2, this.size * 0.2);
		ellipse(this.size * 0.2, this.size * 0.2, this.size * 0.2);
		pop();

		push();
		rotateX(180);
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(-this.size * 0.2, -this.size * 0.3, this.size * 0.2);
		ellipse(-this.size * 0.2, this.size * 0, this.size * 0.2);
		ellipse(-this.size * 0.2, this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.2, -this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.2, this.size * 0, this.size * 0.2);
		ellipse(this.size * 0.2, this.size * 0.3, this.size * 0.2);
		pop();

		push();
		rotateX(270);
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(0, 0, this.size * 0.2);
		ellipse(-this.size * 0.3, -this.size * 0.3, this.size * 0.2);
		ellipse(-this.size * 0.3, this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.3, -this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.3, this.size * 0.3, this.size * 0.2);
		pop();

		push();
		rotateY(90);
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(-this.size * 0.3, -this.size * 0.3, this.size * 0.2);
		ellipse(-this.size * 0.3, this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.3, -this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.3, this.size * 0.3, this.size * 0.2);
		pop();
		
		push();
		rotateY(-90);
		translate(0, 0, this.size * 0.5);
		fill(this.c_bg);
		rect(0, 0, this.size, this.size);
		translate(0, 0, 0.1);
		fill(this.c_dot);
		ellipse(0, 0, this.size * 0.2);
		ellipse(-this.size * 0.3, -this.size * 0.3, this.size * 0.2);
		ellipse(this.size * 0.3, this.size * 0.3, this.size * 0.2);
		pop();
		
		pop();
	}
	
	update () {
		if(this.released) {
			this.vel.x *= REST;
			this.vel.y *= REST;
			this.vel.z += GRAVITY;
			this.pos.x += this.vel.x;
			this.pos.y += this.vel.y;
			this.pos.z += this.vel.z;

			
			if(this.pos.z <= this.height && this.vel.z < 0) {
				this.orientation.x -= this.rot.x % 360;
				this.orientation.y += this.rot.y % 360;
				
				this.rot.x = 0;
				this.rot.y = 0;

				let ox = constrain(map(this.orientation.x % 90, 0, 90, -1, 1), -1, 1);
				let oxsqr = (1 - (ox * ox)) * 0.2;
				
				let oy = constrain(map(this.orientation.y % 90, 0, 90, -1, 1), -1, 1);
				let oysqr = (1 - (oy * oy)) * 0.2;
				
				if(ox > 0) {
 					this.vel.y -= abs(this.vel.z) * oxsqr;
					this.orientation.x += (90 - this.orientation.x % 90) * 0.2;
				} else {
 					this.vel.y += abs(this.vel.z) * oxsqr;
					this.orientation.x -= this.orientation.x % 90 * 0.2;
				}
				
				
				if(oy > 0) {
 					this.vel.x -= abs(this.vel.z) * oysqr;
					this.orientation.y += (90 - this.orientation.y % 90) * 0.2;
				} else {
 					this.vel.x += abs(this.vel.z) * oysqr;
					this.orientation.y -= this.orientation.y % 90 * 0.2;
				}
				
				
				this.jump();
				this.pos.z = this.height;
			}
			
			if(this.pos.x < -width * 0.5) {
				this.pos.x = -width * 0.5;
				this.vel.x *= -1;
			}
			
			if(this.pos.x > width * 0.5) {
				this.pos.x = width * 0.5;
				this.vel.x *= -1;
			}
			
			if(this.pos.y < -height * 0.5) {
				this.pos.y = -height * 0.5;
				this.vel.y *= -1;
			}
			
			if(this.pos.y > height * 0.5) {
				this.pos.y = height * 0.5;
				this.vel.y *= -1;
			}
			
			this.rot.x += this.vel.y * 1.2;
			this.rot.y -= this.vel.x * 1.2;

			
			
			if(this.orientation.x > 360)
				this.orientation.x -= 360;
			if(this.orientation.x < 0)
				this.orientation.x += 360;
			if(this.orientation.y > 360)
				this.orientation.y -= 360;
			if(this.orientation.y < 0)
				this.orientation.y += 360;
// 			print(str(this.orientation.x) + "   " + str(this.orientation.y));
		}
		
	}
	
	jump() {
		this.vel.z *= -BOUNCE;
	}
	
	release() {
		this.released = true;
		this.vel.z += random(0, 20);
	}
	
	drag(_x, _y) {
		if(this.released == false) {
			this.pos.x = _x;
			this.pos.y = _y;
			this.vel.x = (mouseX - pmouseX) * 1.5 + random(-20, 20);
			this.vel.y = (mouseY - pmouseY) * 1.5 + random(-20, 20);;
		}
	}
	
	bounce(other) {
		if(this.distSqr(this.pos, other.pos) < dice_size * dice_size * 1.96) {
			let temp_vx = this.vel.x;
			let temp_vy = this.vel.y;

			this.vel.x = other.vel.x;
			this.vel.y = other.vel.y;

			other.vel.x = temp_vx;
			other.vel.y = temp_vy;
// 			print("bounce");
		}
	}
	
	distSqr(a, b) {
		return (a.x - b.x) * (a.x - b.x) + (a.y - b.y) * (a.y - b.y)
	}
}